-- Source Code Update Script -- v1.3 -----------------------------
IF EXISTS (SELECT 1 FROM INFORMATION_SCHEMA.TABLES WHERE [TABLE_SCHEMA] = 'dbo' AND [TABLE_NAME] = 'asiMappings' AND [TABLE_TYPE] <> 'VIEW')
    DROP TABLE [dbo].[asiMappings] ;

CREATE TABLE [dbo].[asiMappings]([SOURCE_CODE] varchar(40) COLLATE DATABASE_DEFAULT, [Campaign] varchar(10) COLLATE DATABASE_DEFAULT, [SourceCode] varchar(40) COLLATE DATABASE_DEFAULT);

---------------------------------------------------------------------------------------
------------------------------------------------------------------
---- Pre SourceCode Import ------------------------------------
------------------------------------------------------------------
ALTER TABLE [dbo].[CampaignMain] DISABLE TRIGGER ALL;
ALTER TABLE [dbo].[SourceCode] DISABLE TRIGGER ALL;


------------------------------------------------------------------
---- SourceCode Import ------------------------------------
------------------------------------------------------------------
-- STOP HERE!
--
-- USE THE 2-Source Code Import.txt FILE PROVIDED BY THE CLIENT TO
-- RUN THE SOURCE CODE IMPORT BEFORE PROCEEDING.
------------------------------------------------------------------
PRINT '***  STOP HERE!  ***'

---
Insert the source codes into iMIS:
Import the source codes through the staff site after saving the 2-Source Code Import.txt file – Marketing > Campaigns > Import source codes. Verify all imported successfully by clicking drop-down here and seeing no errors.

--------------------------------------------------------------------------------------
------------------------------------------------------------------
---- SourceCode Post Import SQL script ------------------------------------
------------------------------------------------------------------
BEGIN

    SET NOCOUNT ON;

    -- Migrate existing legacy values into new CM values
    UPDATE c
       SET c.[Name] = CASE WHEN c.[Name] = '' THEN lc.[TITLE] ELSE c.[Name] END,
           c.[Description] = CASE WHEN c.[Description] = '' THEN  lc.[DESCRIPTION] ELSE c.[Description] END,
           c.[StartDate] = COALESCE(c.[StartDate], lc.[BEGINS_DATE]),
           c.[EndDate] = COALESCE(c.[EndDate], lc.[ENDS_DATE])
      FROM [dbo].[CampaignMain] c
            INNER JOIN [dbo].[Campaign] lc ON c.[LegacyCampaignCode] = lc.[CAMPAIGN_CODE];

    UPDATE s
       SET s.[Description] = COALESCE(s.[Description], ls.[DESCRIPTION])
      FROM [dbo].[SourceCode] s
            INNER JOIN [dbo].[Appeal] ls ON s.[Code] = ls.[APPEAL_CODE];

    -- Perform the synchronizations that did not occur because the triggers were disabled
    INSERT INTO [dbo].[Gen_Tables] ([CODE], [UPPER_CODE], [DESCRIPTION], [TABLE_NAME])
    SELECT i.[Code], i.[Code], COALESCE(i.[Description],''), 'SOURCE_CODE' 
      FROM [dbo].[SourceCode] i
     WHERE NOT EXISTS (SELECT 1 FROM [dbo].[Gen_Tables] WHERE [TABLE_NAME] = 'SOURCE_CODE' AND [CODE] = i.[Code]);
    
    INSERT INTO [dbo].[Appeal] ([APPEAL_CODE], [TITLE], [DESCRIPTION], [APPEAL_TYPE], [CAMPAIGN_CODE])
    SELECT i.[Code], i.[Code], COALESCE(i.[Description],''), '', c.[LegacyCampaignCode]
      FROM [dbo].[SourceCode] i 
            INNER JOIN [dbo].[SourceCodeTypeRef] sctr ON i.[SourceCodeTypeCode] = sctr.[SourceCodeTypeCode]
            INNER JOIN [dbo].[SolicitationMain] s ON i.[SolicitationKey] = s.[SolicitationKey]
            INNER JOIN [dbo].[AppealMain] a ON s.[AppealKey] = a.[AppealKey]
            INNER JOIN [dbo].[CampaignMain] c ON a.[CampaignKey] = c.[CampaignKey]
     WHERE NOT EXISTS (SELECT 1 FROM [dbo].[Appeal] WHERE [APPEAL_CODE] = i.[Code]);

    INSERT INTO [dbo].[Campaign] ([CAMPAIGN_CODE], [TITLE], [DESCRIPTION], [DEFAULT_FUND], [GOAL], [BEGINS_DATE], [ENDS_DATE], [IS_PERPETUAL])
    SELECT i.[LegacyCampaignCode], i.[Name], COALESCE(i.[Description],''), '', i.[TargetRevenue], i.[StartDate], i.[EndDate], 0 
      FROM [dbo].[CampaignMain] i
     WHERE NOT EXISTS (SELECT 1 FROM [dbo].[Campaign] WHERE [CAMPAIGN_CODE] = i.[LegacyCampaignCode])
END
GO

-- Re-enable the triggers 
ALTER TABLE [dbo].[CampaignMain] ENABLE TRIGGER ALL;
ALTER TABLE [dbo].[SourceCode] ENABLE TRIGGER ALL;


---------------------------------------------------------------------------------------
-- This block needs to be updated with the data from the Updates tab of the 
-- SourceCodesReview Excel workbook after it has been populated according to the 
-- details in the readme file.
---------------------------------------------------------------------------------------
INSERT [dbo].[asiMappings]([SOURCE_CODE], [Campaign], [SourceCode]) 
VALUES 

-- <Paste Here>

;
 

-- -- ***DEBUG*** 
--SELECT * FROM [dbo].[asiMappings];

-------------------------------------------------------------------
-- The following portions of the script will generate SQL to update 
-- all instances of the original SOURCE_CODE with the 
-- script. 
-------------------------------------------------------------------
BEGIN
    CREATE TABLE #asiMappedTables([TABLE_SCHEMA] sysname NOT NULL, [TABLE_NAME] sysname NOT NULL, [RefOrigTransNum] bit, [HasCampaign] bit);

    WITH tablesWithBoth AS (
    SELECT t.[TABLE_NAME], t.[TABLE_SCHEMA] 
      FROM INFORMATION_SCHEMA.COLUMNS c1
            INNER JOIN INFORMATION_SCHEMA.COLUMNS c2 on c1.[TABLE_NAME] = c2.[TABLE_NAME] AND c1.[TABLE_SCHEMA] = c2.[TABLE_SCHEMA]
            INNER JOIN INFORMATION_SCHEMA.TABLES t ON c1.[TABLE_NAME] = t.[TABLE_NAME] AND c1.[TABLE_SCHEMA] = t.[TABLE_SCHEMA]
     WHERE CHARINDEX('SOURCE_CODE', c1.[COLUMN_NAME]) = 1
       AND CHARINDEX('CAMPAIGN', c2.[COLUMN_NAME]) > 0
       AND t.[TABLE_TYPE] <> 'VIEW'
       AND t.[TABLE_NAME] NOT IN ('asiMappings', 'Source_Codes')
    )
    , tablesWithOnlySourceCode AS (
    SELECT t.[TABLE_NAME], t.[TABLE_SCHEMA] 
      FROM INFORMATION_SCHEMA.COLUMNS c1
            LEFT OUTER JOIN INFORMATION_SCHEMA.COLUMNS c2 on c1.[TABLE_NAME] = c2.[TABLE_NAME] AND c1.[TABLE_SCHEMA] = c2.[TABLE_SCHEMA] AND CHARINDEX('CAMPAIGN', c2.[COLUMN_NAME]) > 0
            INNER JOIN INFORMATION_SCHEMA.TABLES t ON c1.[TABLE_NAME] = t.[TABLE_NAME] AND c1.[TABLE_SCHEMA] = t.[TABLE_SCHEMA]
     WHERE CHARINDEX('SOURCE_CODE', c1.[COLUMN_NAME]) = 1
       AND t.[TABLE_TYPE] <> 'VIEW'
       AND c2.[TABLE_NAME] IS NULL
       AND c1.[TABLE_NAME] NOT IN ('Orders','Invoice')
    )
    , addOrdersInvoice AS (
    SELECT t.[TABLE_NAME], t.[TABLE_SCHEMA] 
      FROM INFORMATION_SCHEMA.COLUMNS c1
            LEFT OUTER JOIN INFORMATION_SCHEMA.COLUMNS c2 on c1.[TABLE_NAME] = c2.[TABLE_NAME] AND c1.[TABLE_SCHEMA] = c2.[TABLE_SCHEMA] AND CHARINDEX('CAMPAIGN', c2.[COLUMN_NAME]) > 0
            INNER JOIN INFORMATION_SCHEMA.TABLES t ON c1.[TABLE_NAME] = t.[TABLE_NAME] AND c1.[TABLE_SCHEMA] = t.[TABLE_SCHEMA]
     WHERE CHARINDEX('SOURCE_CODE', c1.[COLUMN_NAME]) = 1
       AND t.[TABLE_TYPE] <> 'VIEW'
       AND c2.[TABLE_NAME] IS NULL
       AND c1.[TABLE_NAME] IN ('Orders','Invoice')
    )
    INSERT #asiMappedTables([TABLE_SCHEMA], [TABLE_NAME], [RefOrigTransNum], [HasCampaign])
    SELECT [TABLE_SCHEMA],
           [TABLE_NAME], 
           0,
           1
      FROM tablesWithBoth
    UNION 
    SELECT [TABLE_SCHEMA],
           [TABLE_NAME], 
           0,
           0
      FROM tablesWithOnlySourceCode
    UNION 
    SELECT [TABLE_SCHEMA],
           [TABLE_NAME], 
           1,
           0
      FROM addOrdersInvoice;

    -- -- ***DEBUG*** 
    --SELECT * FROM #asiMappedTables;

    DECLARE @sql nvarchar(max);

    SELECT @sql = N'';

    SELECT @sql = @sql + N'UPDATE t SET t.[SOURCE_CODE] = m.[SourceCode] FROM [' + mt.[TABLE_SCHEMA] + N'].[' + mt.[TABLE_NAME] + N'] t INNER JOIN [dbo].[asiMappings] m ON t.[SOURCE_CODE] = m.[SOURCE_CODE] INNER JOIN [dbo].[Trans] tr ON t.[ORIGINATING_TRANS_NUM] = tr.[TRANS_NUMBER] AND t.[SOURCE_CODE] = tr.[SOURCE_CODE] AND m.[Campaign] = tr.[CAMPAIGN_CODE];'
      FROM #asiMappedTables mt
     WHERE mt.[RefOrigTransNum] = 1;

    SELECT @sql = @sql + N'UPDATE t SET t.[SOURCE_CODE] = m.[SourceCode], t.[CAMPAIGN_CODE] = m.[Campaign] FROM [' + mt.[TABLE_SCHEMA] + N'].[' + mt.[TABLE_NAME] + N'] t INNER JOIN [dbo].[asiMappings] m ON t.[SOURCE_CODE] = m.[SOURCE_CODE] AND (t.[CAMPAIGN_CODE] = m.[Campaign] OR (t.[CAMPAIGN_CODE] = '''' AND m.[Campaign] = ''Default'')); '
      FROM #asiMappedTables mt
     WHERE mt.[RefOrigTransNum] = 0
       AND mt.[HasCampaign] = 1;

    SELECT @sql = @sql + N'UPDATE t SET t.[SOURCE_CODE] = m.[SourceCode] FROM [' + mt.[TABLE_SCHEMA] + N'].[' + mt.[TABLE_NAME] + N'] t INNER JOIN [dbo].[asiMappings] m ON t.[SOURCE_CODE] = m.[SOURCE_CODE] AND m.[Campaign] = ''Default''; '
      FROM #asiMappedTables mt
     WHERE mt.[RefOrigTransNum] = 0
       AND mt.[HasCampaign] = 0;

    EXEC(@sql);

      -- -- ***DEBUG*** 
        --PRINT @sql; 
        --PRINT SUBSTRING(@sql, 4000, 4000);
        --PRINT SUBSTRING(@sql, 8000, 4000);
        --PRINT SUBSTRING(@sql, 12000, 4000);
        --PRINT SUBSTRING(@sql, 16000, 4000);
        --PRINT SUBSTRING(@sql, 20000, 4000);
        --PRINT SUBSTRING(@sql, 24000, 4000);
        --PRINT SUBSTRING(@sql, 28000, 4000);
        --PRINT SUBSTRING(@sql, 32000, 4000);
        --PRINT SUBSTRING(@sql, 36000, 4000);
        --PRINT SUBSTRING(@sql, 40000, 4000);
        --PRINT SUBSTRING(@sql, 44000, 4000);
        --PRINT SUBSTRING(@sql, 48000, 4000);
        --PRINT SUBSTRING(@sql, 52000, 4000);
        --PRINT SUBSTRING(@sql, 56000, 4000);
        --PRINT SUBSTRING(@sql, 60000, 4000);
        --PRINT SUBSTRING(@sql, 64000, 4000);
        --PRINT SUBSTRING(@sql, 68000, 4000);
        --PRINT SUBSTRING(@sql, 72000, 4000);
        --PRINT SUBSTRING(@sql, 76000, 4000);
        --PRINT SUBSTRING(@sql, 80000, 4000);
        --PRINT SUBSTRING(@sql, 84000, 4000);
        --PRINT SUBSTRING(@sql, 88000, 4000);
        --PRINT SUBSTRING(@sql, 92000, 4000);
        --PRINT SUBSTRING(@sql, 96000, 4000);
        --PRINT SUBSTRING(@sql, 100000, 4000);
        --PRINT SUBSTRING(@sql, 104000, 4000);
        --PRINT SUBSTRING(@sql, 108000, 4000);
        --PRINT SUBSTRING(@sql, 112000, 4000);
        --PRINT SUBSTRING(@sql, 116000, 4000);
        --PRINT SUBSTRING(@sql, 120000, 4000);
        --PRINT SUBSTRING(@sql, 124000, 4000);
        --PRINT SUBSTRING(@sql, 128000, 4000);
        --PRINT SUBSTRING(@sql, 132000, 4000);
        --PRINT SUBSTRING(@sql, 136000, 4000);
        --PRINT SUBSTRING(@sql, 140000, 4000);
        --PRINT SUBSTRING(@sql, 144000, 4000);
        --PRINT SUBSTRING(@sql, 148000, 4000);
        --PRINT SUBSTRING(@sql, 152000, 4000);
        --PRINT SUBSTRING(@sql, 156000, 4000);

END;

DROP TABLE #asiMappedTables;

IF EXISTS (SELECT 1 FROM INFORMATION_SCHEMA.TABLES WHERE [TABLE_SCHEMA] = 'dbo' AND [TABLE_NAME] = 'asiMappings' AND [TABLE_TYPE] <> 'VIEW')
    DROP TABLE [dbo].[asiMappings] ;

GO